/******************************************************************************
** Copyright (C) 2011  Pablo Garcia
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**********************************************************************1********/

// 8 byte access: http://e2e.ti.com/support/microcontrollers/tms320c2000_32-bit_real-time_mcus/f/171/p/21671/83737.aspx#83737

// ----------------------------------------------------------------------------
// Includes
// ----------------------------------------------------------------------------
#include "TIComm.h"


static const char writeCommand[] = "WVAR";		///< write command
static const char readCommand[]  = "RVAR";		///< read command
static const char rtCommand[]    = "CVAR";		///< real time command
static const char delCommand[]   = "DVAR";		///< delete real time command
static const char rstCommand[]   = "RSTM";		///< reset real time memory
static const char startCommand[] = "INIT";		///< starts real time
static const char stopCommand[]  = "STOP";		///< stops real time
struct Commands
{
	Uint16 onWriteCommand:1;
	Uint16 onReadCommand:1;
	Uint16 onRTCommand:1;
	Uint16 onRSTCommand:1;
	Uint16 onDelCommand:1;
	Uint16 onStartCommand:1;
	Uint16 onStopCommand:1;
	Uint16 rsvd:9;
};
union OnCommand
{
	Uint16 all;
	struct Commands command;
};
static const char COMMAND_SIZE = 4;
static const char MSG_SIZE = 12;		//sizeof(struct MsgStruct)*2
static const char RTMSG_SIZE = 8;
volatile union Msg msg;
volatile union Msg dspMsg;
volatile union RTMsg rtMsg;
volatile union OnCommand onCommand;
static struct RTVariables rtVariables[MAX_ACCSESIBLE_VARS];
volatile char usedVariables;
volatile char isrMonitorEnabled=0;
volatile unsigned char RTMonitorStart = 0;

static inline char IsCommand(volatile char origin[], const char command[]);

// ----------------------------------------------------------------------------

void InitTIComm(unsigned char isrMonitor)
{
	__byte((int*)(rtMsg.data.command),0)='R';
	__byte((int*)(rtMsg.data.command),1)='V';
	__byte((int*)(rtMsg.data.command),2)='A';
	__byte((int*)(rtMsg.data.command),3)='R';
	usedVariables = 0;
	ConfigureSCI(SCIARxInterrupt, SCIA, 115200, ONE_STOP_BIT , ODD_PARITY, PARITY_DISABLED, 8);
	isrMonitorEnabled = isrMonitor;
}

// ----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
// ISR on SCIA
// ----------------------------------------------------------------------------
interrupt void SCIARxInterrupt(void)
{
	int cc;
	volatile char needToSend;
	volatile char sciaRXDT;
	volatile char sciaTXDT;
	ToggleGPIOPort(_GPIO18);
	while(SciaRegs.SCIFFRX.bit.RXFFST == 0) {}	// wait for XRDY = 1 for empty state
	sciaRXDT = SciaRegs.SCIRXBUF.bit.RXDT;
	// call protocol
	needToSend=TICommProtocol(sciaRXDT,&dspMsg);
	if(needToSend){
		for(cc=0;cc<MSG_SIZE;cc++){
			sciaTXDT = __byte((int*)(dspMsg.raw),cc);
			SendDataCharSciA(&(sciaTXDT));
		}
			
	}
	SCIARxPIEAck();
}

// ----------------------------------------------------------------------------

// PABLO: Para la parte del monitor en tiempo real, podemos generar una interrupcin
//	cada vez que haya libres 8 bits. En este caso, enviamos el mensaje de una cola.
//	Esta interrupcin slo hay que activarla en el caso de monitor?
interrupt void SCIATxInterrupt(void)
{
	RTMonitor();
	SCIATxPIEAck();
}

// ----------------------------------------------------------------------------


char TICommProtocol(volatile char sciaRXDT, volatile union Msg* dspMsg)
{
	volatile char needToSend = 0;
    static unsigned readedBytes = 0;
	int cc;
	__byte((int*)(msg.raw),readedBytes) = sciaRXDT;
    readedBytes += 1;

    // parse command
	if(readedBytes == COMMAND_SIZE){
		// TODO: Optmize
		if(IsCommand(msg.data.command,writeCommand)){
			onCommand.all=1;
		}
		else if(IsCommand(msg.data.command,readCommand)){
			onCommand.all=2;
		}
		else if(IsCommand(msg.data.command,rtCommand)){
			onCommand.all=4;
		}
		else if(IsCommand(msg.data.command,rstCommand)){
			onCommand.all=8;
		}
		else if(IsCommand(msg.data.command,delCommand)){
			onCommand.all=16;
		}
		else if(IsCommand(msg.data.command,startCommand)){
			onCommand.all=32;
		}
		else if(IsCommand(msg.data.command,stopCommand)){
			onCommand.all=64;
		}
		else {
			onCommand.all=0;
			readedBytes = 0;
		}
	}


    // msg completed
    if(readedBytes == MSG_SIZE){
        readedBytes = 0;
		needToSend = 1;
		// build answer
		*dspMsg = msg;
		if (onCommand.command.onReadCommand) {				// READ COMMAND
			__byte((int*)(dspMsg->data.command),1)='A';
			__byte((int*)(dspMsg->data.command),2)='C';
			__byte((int*)(dspMsg->data.command),3)='K';
			dspMsg->data.val = *((Uint32*)(msg.data.dir));
		}
		else if (onCommand.command.onWriteCommand) {		// WRITE COMMAND
			*((Uint32*)(dspMsg->data.dir)) = dspMsg->data.val;
			dspMsg->data.val = *((Uint32*)(dspMsg->data.dir)); // test. Eliminar?
			// TODO: pensar en construir un comando de ACK por defecto (ackMsg=msg;[0]=codigo)
			__byte((int*)(dspMsg->data.command),1)='A';
			__byte((int*)(dspMsg->data.command),2)='C';
			__byte((int*)(dspMsg->data.command),3)='K';
		}
		else if (onCommand.command.onRTCommand) {			// REAL TIME COMMAND
			needToSend = 1;
			__byte((int*)(dspMsg->data.command),1)='A';
			__byte((int*)(dspMsg->data.command),2)='C';
			__byte((int*)(dspMsg->data.command),3)='K';
			// store the pointer and prepare the real time msg
			if(usedVariables<MAX_ACCSESIBLE_VARS){
				rtVariables[usedVariables].dir = (Uint32*)msg.data.dir;
				rtVariables[usedVariables].pos = usedVariables;
				dspMsg->data.val = rtVariables[usedVariables].pos;
				rtVariables[usedVariables++].used = 1;
			}
		}
		else if (onCommand.command.onDelCommand) {			// DEL MEM COMMAND
			needToSend = 0;
			for (cc=0;cc<usedVariables;cc++){
				if((Uint32)(rtVariables[cc].dir) == msg.data.dir){
					rtVariables[cc].used = 0;
					usedVariables--;
					break;
				}
			}
		}
		else if (onCommand.command.onRSTCommand) {			// RESET MEM COMMAND
			needToSend = 0;
			for (cc=0;cc<usedVariables;cc++)
				rtVariables[cc].used = 0;
			usedVariables = 0;
		}
		else if (onCommand.command.onStartCommand) {		// START REAL TIME COMMAND
			needToSend = 0;
			RTMonitorStart = 1;
			if(isrMonitorEnabled == 1){		
				EnableSCITXInterrupt(SCIA, SCIATxInterrupt, 8);
			}
		}
		else if (onCommand.command.onStopCommand) {			// STOP REAL TIME COMMAND
			needToSend = 0;
			RTMonitorStart = 0;
			if(isrMonitorEnabled == 1){
				DisableSCITXInterrupt(SCIA);
			}
		}
    }
	// test
	return needToSend;
}

// ----------------------------------------------------------------------------

void RTMonitor(void)
{
	static int cc=0;
	int mm;	
	volatile char sciaTXDT;
	if(RTMonitorStart == 0)
		return;
	//for (cc=0;cc<usedVariables;cc++){
	if(cc==usedVariables) cc=0;
		if(rtVariables[cc].used){// creo que sobra esta comprobacion
			__byte((int*)(rtMsg.raw),3) = rtVariables[cc].pos;
			rtMsg.data.val = *((Uint32*)(rtVariables[cc].dir));
			for(mm=0;mm<RTMSG_SIZE;mm++){
				sciaTXDT = __byte((int*)(rtMsg.raw),mm);
				SendDataCharSciANow(&(sciaTXDT));
			}
		cc++;
		}
	//}
}

// ----------------------------------------------------------------------------

static inline char IsCommand(volatile char origin[], const char command[])
{
	int cc;
	char isCommand=1;
	for(cc=0;cc<COMMAND_SIZE;cc++)
		isCommand &= (__byte((int*)(origin),cc) == command[cc]);
	return isCommand;
}

// ----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------
