/******************************************************************************
** Copyright (C) 2007  Pablo Garcia
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/

/**\file**********************************************************************
 File Name	:	TIComm.h
 Project	:  	TIMotorLIB
 
 TI Doc:		na
 \author  		Pablo Garcia (pgarcia@isa.uniovi.es)
 \date 			2011
 \brief 		Communication routines
 \details		This unit allows to communicate the DSP with an external host trough
 the use of SCI port.
******************************************************************************/
	/* protocolo (mensajes al dsp):
    msg: |"WVAR"|dir|val|   --> escribir variable en la dirección dir con el valor val
    msg: |"RVAR"|dir|       --> leer variable en la dirección dir
    msg: |"CVAR"|dir|       --> tiempo real para variable en la dirección dir

        Cuando una variable se establece de tiempo real, se introduce una entrada en la tabla rtVariables con el campo
        used = 1 y el campo dir = dir

    protocolo (mensajes del dsp):
        |"WACK"|dir|val|    --> respuesta tras peticion de escritura de variable
        |"CACK"|dir|0x0000| --> respuesta tras peticion de lectura de variable en tiempo real
        |"RVAR"|dir|val|    --> valor (val) de la variable en la posicion dir

	1) LECTURA VARIABLE:

		remote					dsp

		|"RVAR"|dir|	-->
						<--		|"RACK"|dir|val|

	1) ESCRITURA VARIABLE:

		remote					dsp

		|"WVAR"|dir|val|-->
						<--		|"WACK"|dir|val|

	3) LECTURA TIEMPO REAL:

		remote					dsp

		|"CVAR"|dir|	-->
						<--		|"CACK"|dir|0x0000|

						<--		|"RVAR"|dir|val|
						  .
						  .
						  .
						<--		|"RVAR"|dir|val|

  */

//-----------------------------------------------------------------------------
//	Define
//-----------------------------------------------------------------------------

#ifndef TICOMM_H
#define TICOMM_H

// ----------------------------------------------------------------------------
// Includes
// ----------------------------------------------------------------------------
#include "TIMotorLIB.h"
#include GENERATE_FLEX_INC(Device.h)


typedef struct MsgStruct
{
    char command[2];
    Uint32 dir;
    Uint32 val;
} msgStruct;

union Msg
{
	volatile char raw[6];
	volatile msgStruct data;
};

typedef struct RTMsgStruct
{
	char command[2];
	Uint32 val;
} rtMsgStruct;

union RTMsg
{
	volatile char raw[4];
	volatile rtMsgStruct data;
};

#define MAX_ACCSESIBLE_VARS 16
struct RTVariables{
    unsigned char used;
	unsigned char pos;
    Uint32* dir;
};

extern volatile unsigned char RTMonitorStart;

void InitTIComm(unsigned char isrMonitor);
char TICommProtocol(volatile char sciaRXDT, volatile union Msg* dspMsg);
interrupt void SCIARxInterrupt(void);
interrupt void SCIATxInterrupt(void);
void RTMonitor(void);



#endif

//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------

