/******************************************************************************
** Copyright (C) 2007  Pablo Garcia
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/

/**\file**********************************************************************
 File Name	:	cpuTimers.h
 Project	:  	TIMotorLIB
 
 TI Doc:		<a href="http://www.ti.com/litv/pdf/spru566i"> SPRU566</a> 
 \author  		Pablo Garcia (pgarcia@isa.uniovi.es)
 \date 			2007
 \brief 		CPU Timers
******************************************************************************/

//-----------------------------------------------------------------------------
//	Define
//-----------------------------------------------------------------------------

#ifndef CPUTIMERS_H
#define CPUTIMERS_H

//-----------------------------------------------------------------------------
//	Includes
//-----------------------------------------------------------------------------
#include "TIMotorLIB.h"
#include GENERATE_FLEX_INC(Device.h)

//-----------------------------------------------------------------------------
//	typedefs
// ----------------------------------------------------------------------------
/// enum for selection of desired Timer
typedef enum {
	CPUTIMER0, 	///< CPU Timer 0
	CPUTIMER1, 	///< CPU Timer 1
	CPUTIMER2	///< CPU Timer 2
    } CPUTIMERSEnum;


//-----------------------------------------------------------------------------
//	variables
// ----------------------------------------------------------------------------

/// \struct CPUTIMER_VARSIQ. 
/// \brief Store CpuTimers configuration
struct CPUTIMER_VARSIQ {
   volatile struct  CPUTIMER_REGS  *RegsAddr;	///< pointer to register address
   Uint16   CPUFreqInMHz;			///< CPU clok frequency in Mhz
   Uint32   PeriodInUSec;			///< Timer period in us
};


extern struct CPUTIMER_VARSIQ TIMLCpuTimer0;
// CpuTimer 1 and CpuTimer2 are reserved for DSP BIOS & other RTOS. Comment out CpuTimer1 and CpuTimer2 if using DSP BIOS or other RTOS
extern struct CPUTIMER_VARSIQ TIMLCpuTimer1;
extern struct CPUTIMER_VARSIQ TIMLCpuTimer2;

//-----------------------------------------------------------------------------
//	Implemented Functions
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
///	Initalizes T0, T1, T2 registers to a known state and CPUTIMER_VARSIQ
///	structs.
//-----------------------------------------------------------------------------
void InitCpuTimers(void);

//-----------------------------------------------------------------------------
///	Initalizes selected timer registers to a known state and CPUTIMER_VARSIQ
///	structs. This function is called from ConfigureCpuTimer 
//-----------------------------------------------------------------------------
void InitCpuTimer(CPUTIMERSEnum cpuTimer);

//-----------------------------------------------------------------------------
///	Configure T0, T1 or T2 timer.
///	\param cpuTimer Selected timer (CPUTIMER0, CPUTIMER1, CPUTIMER2)
///	\param Period Timer period in us
///	\param interruptFCN Function to be called on Timer period
//-----------------------------------------------------------------------------
void ConfigureCpuTimer(CPUTIMERSEnum cpuTimer, Uint32 Period,  void(*interruptFCN)(void));


//-----------------------------------------------------------------------------
///	ACK interrupt for Timer 0. This function MUST be called if more interrupts
/// from PIEACK_GROUP1 are wanted
//-----------------------------------------------------------------------------
static inline void Timer0PIEAck(void);


//-----------------------------------------------------------------------------
//	Inline function implementation
//-----------------------------------------------------------------------------
static inline void Timer0PIEAck(void)
{
   	PieCtrlRegs.PIEACK.all |= PIEACK_GROUP1;
}

//-----------------------------------------------------------------------------

#endif


//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------

