/******************************************************************************
** Copyright (C) 2011  Pablo Garcia
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/

// --------------------------------------------------------------------------
// Includes
// --------------------------------------------------------------------------
#include "flash.h"

// Functions that will be run from RAM need to be assigned to
// a different section.  This section will then be mapped to a load and
// run address using the linker cmd file.

#pragma CODE_SECTION(InitFlash, "ramfuncs");

// This function initializes the Flash Control registers

//                   CAUTION
// This function MUST be executed out of RAM. Executing it
// out of OTP/Flash will yield unpredictable results

//---------------------------------------------------------------------------

void InitFlash(void)
{
	EALLOW;
   	//Enable Flash Pipeline mode to improve performance
   	//of code executed from Flash.
   	FlashRegs.FOPT.bit.ENPIPE = 1;

   	//                CAUTION
   	//Minimum waitstates required for the flash operating
   	//at a given CPU rate must be characterized by TI.
   	//Refer to the datasheet for the latest information.
	#if CPU_FRQ_150MHZ
   		//Set the Paged Waitstate for the Flash
   		FlashRegs.FBANKWAIT.bit.PAGEWAIT = 5;

   		//Set the Random Waitstate for the Flash
   		FlashRegs.FBANKWAIT.bit.RANDWAIT = 5;

   		//Set the Waitstate for the OTP
   		FlashRegs.FOTPWAIT.bit.OTPWAIT = 8;
	#endif

	#if CPU_FRQ_100MHZ
   		//Set the Paged Waitstate for the Flash
   		FlashRegs.FBANKWAIT.bit.PAGEWAIT = 3;

   		//Set the Random Waitstate for the Flash
   		FlashRegs.FBANKWAIT.bit.RANDWAIT = 3;

   		//Set the Waitstate for the OTP
   		FlashRegs.FOTPWAIT.bit.OTPWAIT = 5;
	#endif
	//                CAUTION
	//ONLY THE DEFAULT VALUE FOR THESE 2 REGISTERS SHOULD BE USED
	FlashRegs.FSTDBYWAIT.bit.STDBYWAIT = 0x01FF;
	FlashRegs.FACTIVEWAIT.bit.ACTIVEWAIT = 0x01FF;
	EDIS;

	//Force a pipeline flush to ensure that the write to
   	//the last register configured occurs before returning.

   	asm(" RPT #7 || NOP");
}

//---------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------
