/******************************************************************************
** Copyright (C) 2007  Pablo Garcia
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/

#ifndef MOTORCONSTANTS_H
#define MOTORCONSTANTS_H

#include "IQConstants.h"

#define SPMSM_ABB			// Surface permanent magnet motor ABB

#ifdef IM
	#ifdef DS
	#undef DS
	#endif
	#ifdef MX
	#undef MX
	#endif
	#define QUADRATURE_AXIS_REAL
	#define MOTORDEFINED
	//#warning Induction Machine is selected
#endif

#define DS		// delta systems IPMS
#ifdef DS
	#ifdef IM
	#undef IM
	#endif
	#ifdef MX
	#undef MX
	#endif
	#define DIRECT_AXIS_REAL
	#define MOTORDEFINED
	//#warning Delta System machine is selected
#endif

//#define MX		// Maxom IPMS
#ifdef MX
	#ifdef IM
	#undef IM
	#endif
	#ifdef DS
	#undef DS	
	#endif
	#define DIRECT_AXIS_REAL
	#define MOTORDEFINED
	//#warning Maxon Machine is selected
#endif

#ifndef MOTORDEFINED
	#error Motor is not defined
#endif

#define RS					(12)				// stator resistance [ohms]
#define RR					(12)				// rotor resistance [ohms]
#define LM					(_IQ(0.0005))		// mutual inductance [henry]
#define LLS					(_IQ(0.00006))		// stator leakage inductance [henry]
#define LLR					(_IQ(0.00006))		// rotor leakage inductance [henry]
#define LR					(_IQ(0.06))			// rotor inductance [henry]
#define LS  				(_IQ(0.0144))		// stator inductance [H]
#define TAUS				(_IQ(0.0012))		// stator time constants
#define INVTAUS				(_IQ(833.334))		// inverse stator time constants

#ifdef IM
   	#define P   			(2)					//number of pole pairs
	#define R				28					// Rotor bars
	#define S				36					// Stator slots						
	#define I_LIMIT   		(_IQ(20))			//integrator limit [V]
	#define V_LIMIT   		(_IQ(150))			//command voltage limit (per phase +/-) [V]
	#define ISTAR_LIMIT		(_IQ(120))			//current command limit for flux axis
	//#define VBUS      		(_IQ(300))			//DC Bus Voltage (Battery Voltage) [V]
	#define INV_VBUS		(_IQ(0.00333))
	#define I0				(_IQ(2.4))			// no load current
	#define Ld				(_IQ(0.00082))
	#define Lq				(_IQ(0.00082))
	#define TAUS			(_IQ(0.0012))		// stator time constants
	#define INVTAUS			(_IQ(833.334))		// inverse stator time constants
	#define TR				(_IQ(0.0872756933))	// rotot time constant
	#define F_LINK			(_IQ(0.00265258))
	#define SALIENCY_ORDER	(_IQ(14))			// rotor saliency order
#endif

#ifdef DS										  
	#define P   			(2)					//number of pole pairs
	//#define RS_DS			(_IQ(3.1))			//stator resistance [ohms]
	//#define LS_DS  		(_IQ(0.005))		//stator inductance [H]			
	//#define I_LIMIT   	(_IQ(7.6))			//integrator limit [V]						
	#define V_LIMIT   		(_IQ(70))			//command voltage limit (per phase +/-) [V]
	//#define ISTAR_LIMIT	(_IQ(120))			//current command limit for flux axis
	//#define VBUS      		(_IQ(198))			//DC Bus Voltage (Battery Voltage) [V]
	#define INV_VBUS		(_IQ(0.00505050))
	#define I0				(_IQ(2.68)) 		// no load current
	#define J				(_IQ(0.0015))		//  intertia (kg*m^2)
	//#define INVJ				(_IQ17(12033.6943))	
	#define Ld				(_IQ(0.00046))		//@2500 Hz
	#define Lq				(_IQ(0.00055))		//@2500 Hz
	#define Rd				(_IQ(11))			//@2500 Hz
	#define Rq				(_IQ(16))			//@2500 Hz
	#define F_LINK			(_IQ(0.1378))
	#define TR				(_IQ(0))			// rotot time constant (CHANGE!!!!)	
	#define E_LINES			(2000)
	#define SALIENCY_ORDER	(_IQ(2))			// rotor saliency order
#endif

#ifdef MX
	#define P   				(1)				//number of pole pairs	
	#define I_LIMIT   			(_IQ(10))		//integrator limit [V]				
	#define V_LIMIT   			(_IQ(25))		//integrator limit [V]				
	#define ISTAR_LIMIT			(_IQ(15))		//current command limit for flux axis				
	//#define VBUS      			(_IQ(90))				
	#define INV_VBUS			(_IQ(0.0111111111111111111))
	#define I0					(_IQ(5.0))		// no load current
	//#define J		(_IQ(0.0000831))			//  intertia (kg*m^2)
	#define J					(_IQ(0.0035))	//  intertia (kg*m^2)
	#define INVJ				(_IQ17(12033.6943))
	#define Ld					(_IQ(0.00031)) 	// 0.00031
	#define Lq					(_IQ(0.00035)) 	// 0.00035
	#define Rd					(_IQ(3.6))
	#define Rq					(_IQ(3.6))
	#define F_LINK				(_IQ(0.115))
	#define TR					(_IQ(0))		// rotot time constant (CHANGE!!!!)	
	#define E_LINES				(2000)
   #define SALIENCY_ORDER		(_IQ(2))		// rotor saliency order
#endif
#endif
