/******************************************************************************
** Copyright (C) 2010  
** Silvia Izquierdo Rosas <silvia.izquierdo.rosas@gmail.com>
** Christian Blanco Charro <khristian996@gmail.com>
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/

// ----------------------------------------------------------------------------
// Includes
// ----------------------------------------------------------------------------
#include "TIMotorLIB.h"
#include "DataStorage.h"
#include "sci.h"

// ----------------------------------------------------------------------------

void ConfigureSCI(void(*interruptFCN)(void), 
	SCI_PORT sciPort, Uint32 speed, SCI_STOPBIT stopBit , SCI_PARITY parity, 
	SCI_PARITY_ENABLE sciParityEnable, Uint16 nbit)

{
	volatile struct SCI_REGS* sciRegs;
	Uint16 BRR;
	if(sciPort == SCIA) {
		sciRegs = &SciaRegs;
		EALLOW;
		SysCtrlRegs.PCLKCR0.bit.SCIAENCLK = 1;   // Clocks were turned on to the SCIA peripheral
		EDIS;
	}
	else if(sciPort == SCIB){
		sciRegs = &ScibRegs;		
		EALLOW;
		SysCtrlRegs.PCLKCR0.bit.SCIBENCLK = 1;   // Clocks were turned on to the SCIB peripheral
		EDIS;
	}
	InitSCIGpio(sciPort);
   	SciFifoInit(sciRegs);	   // Initialize the SCI FIFO


	if(interruptFCN != 0) {
		if(sciPort == SCIA){
    		EALLOW;
			PieVectTable.SCIRXINTA = interruptFCN;
			EDIS;					
			PieCtrlRegs.PIEIER9.bit.INTx1=1;     // PIE Group 9, int1 SCIRXINTA
			IER |= M_INT9;	// Enable CPU INT
		}
		else if(sciPort == SCIB){
    		EALLOW;
			PieVectTable.SCIRXINTB = interruptFCN;
			EDIS;					
			PieCtrlRegs.PIEIER9.bit.INTx3=1;     // PIE Group 9, int3 SCIRXINTB
			IER |= M_INT9;	// Enable CPU INT
		}
   		EINT;
	}
	// SCICCR
	sciRegs->SCICCR.bit.STOPBITS = stopBit;
	sciRegs->SCICCR.bit.PARITY = parity;
	sciRegs->SCICCR.bit.PARITYENA = sciParityEnable;
	sciRegs->SCICCR.bit.LOOPBKENA = 0;	// disable loopback
	sciRegs->SCICCR.bit.ADDRIDLE_MODE = 0; //Idle line mode protocol 
	sciRegs->SCICCR.bit.SCICHAR = nbit-1;
	// SCICTL1
	sciRegs->SCICTL1.bit.RXERRINTENA = 0; //Receive error interrupt disabled
	sciRegs->SCICTL1.bit.SWRESET = 0; //Initializes SCI state machines and operating flags to the reset condition
	sciRegs->SCICTL1.bit.TXWAKE = 0; //Transmit feature is not selected
	sciRegs->SCICTL1.bit.SLEEP = 0; //Sleep mode disabled
	sciRegs->SCICTL1.bit.TXENA = 1; //Transmitter enabled
	sciRegs->SCICTL1.bit.RXENA = 1; //Send received character to SCIRXEMU and SCIRXBUF
	// SCICTL2
	sciRegs->SCICTL2.bit.RXBKINTENA =1; // Enable RXRDY/BRKDT interrupt

	// Port speed configuration: spru051b.pdf --> pag. 28
	// if BRB = 0 --> Baud rate = LSPCLK/16
	BRR = (Uint32)(_IQtoF(LspCLKInMhz())*1000000)/(speed << 3) - 1;
	sciRegs->SCIHBAUD = (BRR & 0xFF00) >> 8;
	sciRegs->SCILBAUD = (BRR & 0x00FF);

	sciRegs->SCIFFRX.bit.RXFIFORESET = 1; //Re-enable receive FIFO operation
	sciRegs->SCIFFTX.bit.TXFIFOXRESET = 1; //Re-enable transmit FIFO operation
	sciRegs->SCICTL1.bit.SWRESET = 1; //After a system reset, re-enable the SCI by writing a 1 to this bit

}

// ----------------------------------------------------------------------------

void SciFifoInit(volatile struct SCI_REGS* sciRegs)
{
	//SCIFFTX = 0xE040		spru051b.pdf --> pag. 34
	sciRegs->SCIFFTX.bit.SCIRST = 1; //SCI can transmit or receive
	sciRegs->SCIFFTX.bit.SCIFFENA = 1; //SCI FIFO enhancements enabled
	sciRegs->SCIFFTX.bit.TXFIFOXRESET = 0; //Re-enable transmit FIFO operation
	sciRegs->SCIFFTX.bit.TXFFINTCLR = 1; //To clear TRFFINT flag in bit 7 --NO CAMBIA--
	sciRegs->SCIFFTX.bit.TXFFIENA = 0; //Interrupt based on TXFFIVL disabled
	sciRegs->SCIFFTX.bit.TXFFIL = 0; //Transmit FIFO level bits

	//SCIFFRX = 0x0021		spru051b.pdf --> pag. 35
	sciRegs->SCIFFRX.bit.RXFFOVRCLR = 1; //Has no effect on RXFFOVF, bit reads back a zero
	sciRegs->SCIFFRX.bit.RXFIFORESET = 0; //Reset the FIFO pointer to zero, and hold in reset
	sciRegs->SCIFFRX.bit.RXFFINTCLR = 1; //Has no effect on RXFFOVF, bit reads back a zero
	sciRegs->SCIFFRX.bit.RXFFIENA = 1; //RX FIFO interrupt based on RXFFIVL match will be enabled
	sciRegs->SCIFFRX.bit.RXFFIL = 1; //Receive FIFO interrupt level bits


	//SCIFFCT = 0x0;		spru051b.pdf --> pag. 36
	sciRegs->SCIFFCT.bit.ABDCLR = 0; //Has no effect on ABD flag bit
	sciRegs->SCIFFCT.bit.CDC = 0; //Disables auto-baud alignment
	sciRegs->SCIFFCT.bit.FFTXDLY = 0; //FIFO transfer delay
}

// ----------------------------------------------------------------------------

void InitSCIGpio(SCI_PORT sciPort)
{
	// TODO: GPIO port selection depends on the particular hardware --> configurable
	if(sciPort == SCIA) { // RX: 28/36, TX: 29,35
		EALLOW;
		GpioCtrlRegs.GPAPUD.bit.GPIO28 = 0;    // Enable pull-up for GPIO28 (SCIRXDA)
		GpioCtrlRegs.GPAPUD.bit.GPIO29 = 0;	   // Enable pull-up for GPIO29 (SCITXDA)
		GpioCtrlRegs.GPAQSEL2.bit.GPIO28 = 3;  // Asynch input GPIO28 (SCIRXDA)
		GpioCtrlRegs.GPAMUX2.bit.GPIO28 = 1;   // Configure GPIO28 for SCIRXDA operation
		GpioCtrlRegs.GPAMUX2.bit.GPIO29 = 1;   // Configure GPIO29 for SCITXDA operation
    	EDIS;

		/*EALLOW;    
		GpioCtrlRegs.GPBPUD.bit.GPIO35 = 0;    // Enable pull-up for GPIO35 (SCITXDA)
		GpioCtrlRegs.GPBPUD.bit.GPIO36 = 0;	   // Enable pull-up for GPIO29 (SCIRXDA)
		GpioCtrlRegs.GPBQSEL1.bit.GPIO36 = 3;  // Asynch input GPIO36 (SCIRXDA)
		GpioCtrlRegs.GPBMUX1.bit.GPIO35 = 1;   // Configure GPIO35 for SCITXDA operation
		GpioCtrlRegs.GPBMUX1.bit.GPIO36 = 1;   // Configure GPIO36 for SCIRXDA operation
	    EDIS;*/

	}
	else if(sciPort == SCIB) { // RX: 11/15/19/23, TX: 9/14/18/22
		EALLOW;    
		// TODO:
		EDIS;
	}
}

void EnableSCITXInterrupt(SCI_PORT sciPort, void(*interruptFCN)(void), unsigned char fifoLevel)
{
	volatile struct SCI_REGS* sciRegs;
	if(sciPort == SCIA){
		sciRegs = &SciaRegs;
		EALLOW;
		PieVectTable.SCITXINTA = interruptFCN;
		EDIS;					
		PieCtrlRegs.PIEIER9.bit.INTx2=1;     // PIE Group 9, int2 SCITXINTA
		IER |= M_INT9;	// Enable CPU INT
	}
	else if(sciPort == SCIB){
		sciRegs = &ScibRegs;
		EALLOW;
		PieVectTable.SCITXINTB = interruptFCN;
		EDIS;					
		PieCtrlRegs.PIEIER9.bit.INTx4=1;     // PIE Group 9, int4 SCITXINTB
		IER |= M_INT9;	// Enable CPU INT
	}
	EINT;

	sciRegs->SCIFFTX.bit.TXFFIENA = 1; //Interrupt based on TXFFIVL enabled
	sciRegs->SCIFFTX.bit.TXFFIL = 16-fifoLevel; //Transmit FIFO level bits
	sciRegs->SCIFFTX.bit.TXFIFOXRESET = 1; //Re-enable transmit FIFO operation
	sciRegs->SCICTL1.bit.SWRESET = 1; //After a system reset, re-enable the SCI by writing a 1 to this bit
}

// ----------------------------------------------------------------------------

void DisableSCITXInterrupt(SCI_PORT sciPort)
{
	volatile struct SCI_REGS* sciRegs;
	if(sciPort == SCIA){
		sciRegs = &SciaRegs;
	}
	else if(sciPort == SCIB){
		sciRegs = &ScibRegs;
	}
	sciRegs->SCIFFTX.bit.TXFFIENA = 0; //Interrupt based on TXFFIVL disabled
	sciRegs->SCIFFTX.bit.TXFFIL = 0; //Transmit FIFO level bits
}

// ----------------------------------------------------------------------------

/*
// TODO: Pablo: Sería mejor almacenar el indice de leidas en la unidad y que el
// usuario se despreocupara de ellas
int AddVar2List(char *name,void *pointer,int readedInx,struct VAR_DESC *variables)
{
	int i;

	if(readedInx<TAM_LISTA_VAR){
		for(i=0;i<TAM_NOMBRE_VAR;i++)
			*variables[readedInx].VarName=name[i];

		variables[readedInx].PVar=pointer;
		return 1;
	}
	return 0;
}

// ----------------------------------------------------------------------------

void ResetVector(Uint16* begin, Uint16* end)
{	
	Uint16* i;

    for(i=begin;i<end;i++)
        *i=0;

}

// ----------------------------------------------------------------------------
*/

//-----------------------------------------------------------------------------
// End Of File
//----------------------------------------------------------------------------- 
