/******************************************************************************
** Copyright (C) 2010  
** Pablo Garca Fernndez <pgarcia@isa.uniovi.es>
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/

// ----------------------------------------------------------------------------
// Includes
// ----------------------------------------------------------------------------
#include "TIMotorLIB.h"
#include "spi.h"

void SpiFifoInit(void);

void ConfigureSPIA(void(*interruptRX)(void), 
	void(*interruptTX)(void), SPINMC networkModeControl,
	SPILBK spiLoopBack, Uint32 speed, Uint16 SPIcharLength, Uint16 txFIFO, Uint16 rxFIFO)
{
	Uint16 spiIntEna=0;
	Uint16 brr;
	Uint32 lspCLK;
	EALLOW;
	SysCtrlRegs.PCLKCR0.bit.SPIAENCLK = 1;   // Clocks were turned on to the SPIA peripheral
	EDIS;
	InitSpiaGpio();
	//SpiFifoInit();
	// Initialize SPI FIFO registers
   	SpiaRegs.SPICCR.bit.SPISWRESET=0; 			// Reset SPI
	SpiaRegs.SPICCR.bit.SPILBK = spiLoopBack;	// SPI lookback
	SpiaRegs.SPICCR.bit.SPICHAR = SPIcharLength-1;
	// Operation Control Register
	SpiaRegs.SPICTL.bit.MASTER_SLAVE = networkModeControl;
	SpiaRegs.SPICTL.bit.TALK = 1;
	SpiaRegs.SPISTS.all=0x0000;					// reset status register
	SpiaRegs.SPIFFTX.bit.SPIRST = 1;			// reset transmit/receive chanels
	SpiaRegs.SPIFFTX.bit.SPIFFENA = 1;			// enhancements enabled
	SpiaRegs.SPIFFTX.bit.TXFFIL = txFIFO;		// Transmission FIFO level
	SpiaRegs.SPIFFRX.bit.RXFFIL = rxFIFO;		// Receive FIFO level
	SpiaRegs.SPIFFRX.bit.RXFFOVFCLR = 1;		// Clear RX overflow flag
	SpiaRegs.SPIFFCT.all=0x00;					// FIFO transmit delay
	SpiaRegs.SPIPRI.all=0x0010;
	SpiaRegs.SPIPRI.bit.FREE = 1;                // Set so breakpoints don't disturb xmission
   	EALLOW;	// This is needed to write to EALLOW protected registers
	if(interruptRX != 0) {
		spiIntEna=1;
		SpiaRegs.SPIFFRX.bit.RXFFIENA=1;
	   	PieVectTable.SPIRXINTA = interruptRX;
		PieCtrlRegs.PIEIER6.bit.INTx1=1;     // Enable PIE Group 6, INT 1
	}
	if(interruptTX != 0) {
		spiIntEna=1;
		SpiaRegs.SPIFFTX.bit.TXFFIENA=1;
	   	PieVectTable.SPITXINTA = interruptTX;
		PieCtrlRegs.PIEIER6.bit.INTx2=1;     // Enable PIE Group 6, INT 2
	}
	if(spiIntEna){
		SpiaRegs.SPICTL.bit.SPIINTENA = 1;
		IER |= M_INT6;	// Enable CPU INT
	}
   	EDIS;   // This is needed to disable write to EALLOW protected registers


	// Port speed configuration: sprueu3a.pdf --> pag. 18
	// if BRR = 0, 1, 2 --> Baud rate = LSPCLK/4
	lspCLK = (Uint32)(_IQtoF(LspCLKInMhz()));
	if(speed >= (lspCLK>>2))
		brr = 0;
	else
		brr = lspCLK/speed - 1;
	//brr = 9;
	SpiaRegs.SPIBRR = brr;

	StartTransmission();
}

// ----------------------------------------------------------------------------

void InitSpiaGpio(void)
{
	EALLOW;
	// Enable internal pull-up for the selected pins
    GpioCtrlRegs.GPAPUD.bit.GPIO16 = 0;   // Enable pull-up on GPIO16 (SPISIMOA)
    GpioCtrlRegs.GPAPUD.bit.GPIO17 = 0;   // Enable pull-up on GPIO17 (SPISOMIA)
    GpioCtrlRegs.GPAPUD.bit.GPIO18 = 0;   // Enable pull-up on GPIO18 (SPICLKA)
    GpioCtrlRegs.GPAPUD.bit.GPIO19 = 0;   // Enable pull-up on GPIO19 (SPISTEA)

	/*GpioCtrlRegs.GPBPUD.bit.GPIO54 = 0;   // Enable pull-up on GPIO54 (SPISIMOA)
	GpioCtrlRegs.GPBPUD.bit.GPIO55 = 0;   // Enable pull-up on GPIO55 (SPISOMIA)
	GpioCtrlRegs.GPBPUD.bit.GPIO56 = 0;   // Enable pull-up on GPIO56 (SPICLKA)
	GpioCtrlRegs.GPBPUD.bit.GPIO57 = 0;   // Enable pull-up on GPIO57 (SPISTEA)
	*/
	// Set qualification for selected pins to asynch only
    GpioCtrlRegs.GPAQSEL2.bit.GPIO16 = 3; // Asynch input GPIO16 (SPISIMOA)
    GpioCtrlRegs.GPAQSEL2.bit.GPIO17 = 3; // Asynch input GPIO17 (SPISOMIA)
    GpioCtrlRegs.GPAQSEL2.bit.GPIO18 = 3; // Asynch input GPIO18 (SPICLKA)
    GpioCtrlRegs.GPAQSEL2.bit.GPIO19 = 3; // Asynch input GPIO19 (SPISTEA)
	
	/*GpioCtrlRegs.GPBQSEL2.bit.GPIO54 = 3; // Asynch input GPIO16 (SPISIMOA)
	GpioCtrlRegs.GPBQSEL2.bit.GPIO55 = 3; // Asynch input GPIO17 (SPISOMIA)
	GpioCtrlRegs.GPBQSEL2.bit.GPIO56 = 3; // Asynch input GPIO18 (SPICLKA)
	GpioCtrlRegs.GPBQSEL2.bit.GPIO57 = 3; // Asynch input GPIO19 (SPISTEA)
	*/
	// Configure SPI-A pins using GPIO regs*
    GpioCtrlRegs.GPAMUX2.bit.GPIO16 = 1; // Configure GPIO16 as SPISIMOA
    GpioCtrlRegs.GPAMUX2.bit.GPIO17 = 1; // Configure GPIO17 as SPISOMIA
    GpioCtrlRegs.GPAMUX2.bit.GPIO18 = 1; // Configure GPIO18 as SPICLKA
	GpioCtrlRegs.GPAMUX2.bit.GPIO19 = 1; // Configure GPIO19 as SPISTEA
	
	/*GpioCtrlRegs.GPBMUX2.bit.GPIO54 = 1; // Configure GPIO54 as SPISIMOA
	GpioCtrlRegs.GPBMUX2.bit.GPIO55 = 1; // Configure GPIO55 as SPISOMIA
	GpioCtrlRegs.GPBMUX2.bit.GPIO56 = 1; // Configure GPIO56 as SPICLKA
	GpioCtrlRegs.GPBMUX2.bit.GPIO57 = 1; // Configure GPIO57 as SPISTEA
	*/
	EDIS;
}

// ----------------------------------------------------------------------------

void SpiFifoInit(void)
{
	// Initialize SPI FIFO registers
   	SpiaRegs.SPICCR.bit.SPISWRESET=0; // Reset SPI

	SpiaRegs.SPICCR.all=0x001F;       //16-bit character, Loopback mode
	SpiaRegs.SPICTL.all=0x0017;       //Interrupt enabled, Master/Slave XMIT enabled
	SpiaRegs.SPISTS.all=0x0000;
	SpiaRegs.SPIBRR=0x0063;           // Baud rate
	SpiaRegs.SPIFFTX.all=0xC028;      // Enable FIFO's, set TX FIFO level to 8
	SpiaRegs.SPIFFRX.all=0x0028;      // Set RX FIFO level to 8
	SpiaRegs.SPIFFCT.all=0x00;
	SpiaRegs.SPIPRI.all=0x0010;

	SpiaRegs.SPICCR.bit.SPISWRESET=1;  // Enable SPI

	SpiaRegs.SPIFFTX.bit.TXFIFO=1;
	SpiaRegs.SPIFFRX.bit.RXFIFORESET=1;
}

// ----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// End Of File
//----------------------------------------------------------------------------- 
